/* Copyright 2022 Matthew J Wolf
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * -----
 * Based on F62_-_HHKB_-_Split_Backspace.json from Joe of Model F Labs
 */

#include QMK_KEYBOARD_H

enum layer_names {
    _BASE,
    _CAP,
    _FN1,
    _NUM
};

enum custom_keycodes {
  TBMACRO = SAFE_RANGE,
  DGRMCRO,
  WRKMOD
};

bool bnumlock = false;
bool bcapslock = false;
bool bscrollock = false;
bool numlock_changed = false;
bool workmode = true;


const uint16_t PROGMEM keymaps[][MATRIX_ROWS][MATRIX_COLS] = {
    [_BASE] = LAYOUT_all(
        LSFT(KC_9),   KC_6,    KC_7,    KC_8,   KC_9,      KC_0,   KC_MINS,
        RALT(KC_9),   KC_Y,    KC_U,    KC_I,   KC_O,      KC_P,   KC_LBRC,
                    KC_H,   KC_J,    KC_K,   KC_L,      KC_SCLN,KC_QUOT,
        RALT(KC_0),    KC_N,   KC_M, KC_COMM, KC_DOT,      KC_SLSH,KC_ENT,
                            MO(_FN1), KC_BSPC, KC_RALT, KC_NUHS, KC_ESC,
        KC_PGDN,    KC_PGUP,
        KC_HOME,    KC_SPC, KC_BSPC,
        KC_END, KC_SPC, KC_BSPC
    ),
    [_CAP] = LAYOUT_all(
        KC_5,    KC_6,    KC_7,    KC_8,    KC_9,    KC_0,    KC_MINS,
        KC_CAPS,  KC_ESC, KC_PGUP,   KC_UP, KC_PGDN, KC_PSCR, KC_BSPC,
                KC_HOME, KC_LEFT, KC_DOWN, KC_RGHT, _______,  KC_DEL,
        _______,KC_END, _______, _______, _______, LSFT(KC_MINS), _______,
                        _______, _______, _______, LSFT(KC_1), _______,
        _______, _______,
        _______, _______, _______,
        QK_BOOT, _______, _______
    ),
    [_FN1] = LAYOUT_all(
        KC_5,    KC_6,      KC_7,    KC_8,    KC_9,    KC_0,    KC_MINS,
        LSFT(KC_5),   LSFT(KC_6),        LSFT(KC_7),      LSFT(KC_8),   LSFT(KC_9),       LSFT(KC_0),       LSFT(KC_MINS),
                KC_RBRC, RALT(KC_7), RALT(KC_8), RALT(KC_9),    RALT(KC_0), RALT(KC_MINS),
        _______, _______,   RALT(KC_M),   KC_RBRC,  S(KC_RBRC), RALT(KC_RBRC),       _______,
                            _______,   _______,     _______,       _______, _______,
        _______, _______,
        _______, _______, _______,
        QK_BOOT, _______, _______
    ),
    [_NUM] = LAYOUT_all(
        KC_PERC, KC_CIRC, KC_AMPR, KC_ASTR, KC_LPRN, KC_RPRN, KC_BSPC,
        KC_ESC, KC_BSPC, KC_PSLS,  KC_P7,  KC_P8,  KC_P9, KC_PMNS,
                KC_HOME, KC_PGUP,  KC_P4,  KC_P5,  KC_P6, KC_PPLS,
        KC_DEL, KC_END, KC_PGDN,  KC_P1,  KC_P2,  KC_P3, KC_PENT,
                            KC_PAST,  KC_P0,  KC_PDOT, KC_DOT, KC_ENT,
        _______, _______,
        _______, _______, _______,
        QK_BOOT, _______, _______
    )
};


bool process_record_user(uint16_t keycode, keyrecord_t *record) {
    if (record->event.pressed) {
        switch(keycode) {
            case TBMACRO:
                //SEND_STRING(SS_TAP(X_TAB) SS_DOWN(X_LSHIFT) SS_TAP(X_HOME) SS_UP(X_LSHIFT) SS_TAP(X_DELETE));
                tap_code(KC_TAB);
                register_code(KC_LSFT);
                tap_code(KC_HOME);
                unregister_code(KC_LSFT);
                tap_code(KC_DEL);
                return false;
            case DGRMCRO:
                if(!bnumlock) {
                    //register_code(KC_NLCK);
                    //unregister_code(KC_NLCK);
                    tap_code(KC_NUM);
                    bnumlock = true;
                }
                //SEND_STRING(SS_DOWN(X_LALT) SS_TAP(X_P0) SS_TAP(X_P1) SS_TAP(X_P7) SS_TAP(X_P6) SS_UP(X_LALT));
                //SEND_STRING(SS_DOWN(X_LALT) SS_TAP(X_KP_0) SS_TAP(X_KP_1) SS_TAP(X_KP_7) SS_TAP(X_KP_6) SS_UP(X_LALT) );
                register_code(KC_LALT);
                tap_code(KC_KP_0);
                tap_code(KC_KP_1);
                tap_code(KC_KP_7);
                tap_code(KC_KP_6);
                unregister_code(KC_LALT);
                return false;       
        }
        
    }
    return true;
}

/*
layer_state_t layer_state_set_user(layer_state_t state) {
    switch (biton32(state)) {
        case _BASE:
            break;
        case _NUM:
            break;
        default: //  for any other layers, or the default layer   
            break;
    }
  return state;
}
*/


void led_set_user(uint8_t usb_led){
    if (host_keyboard_led_state().num_lock & !bnumlock) {
        layer_on(3);
        bnumlock = true;

    } else if (!host_keyboard_led_state().num_lock & bnumlock) {
        layer_off(3);
        bnumlock = false;
    }
    if (host_keyboard_led_state().caps_lock & !bcapslock) {
        layer_on(1);
        bcapslock = true;
    } else if (!host_keyboard_led_state().caps_lock & bcapslock) {
        layer_off(1);
        bcapslock = false;
    }
    if (host_keyboard_led_state().scroll_lock & !bscrollock){
        bscrollock = true;
    } else if (bscrollock){
        bscrollock = false;
    }
}